__author__  = 'Oberon IT <alle@oberon.no>'

from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Acquisition import aq_parent

from Marshall import ScoutRFC822Marshaller
from Translatable import Translatable

from config import PROJECTNAME
#from config import VIEW_CONTENT_PERMISSION

schema = BaseSchema + Schema((
    TextField('comment',
              required=0,
              searchable=1,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = TextAreaWidget(description="""Describe the listing"""),
              	rows=3, cols=70),
    TextField('generator',
              required=1,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = StringWidget(description="""Python expression that generates the list of objects to list""",
		size=70),
              ),
    TextField('buttons',
              required=0,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = StringWidget(description="""Name of portal_action group listing buttons to display""",
		size=70),
              ),
    TextField('type_name',
              required=0,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = StringWidget(description="""Object type that the 'Add' button (if any) for this listing should create.""",
		size=70),
              ),
    TextField('object_folder',
              required=0,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = StringWidget(description="""Relative path to folder that objects should be created in (is possible)""",
		size=70),
              ),
    TextField('categories',
              required=0,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
#              widget = TextAreaWidget(description="""List of python expressions to extract categories, one per line""",
              widget = TextAreaWidget(description="""Python expression that categorizes the listing""",
		rows=6, cols=70),
              ),
    TextField('headers',
              required=1,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = TextAreaWidget(description="""Header titles, one per line, with leading column number""",
		rows=6, cols=70),
              ),
    TextField('columns',
              required=1,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = TextAreaWidget(description="""Python expressions for generating column values, one per line, with leading column number""",
		rows=6, cols=70),
              ),
    IntegerField('sortby',
              required=1,
	      default=1,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = IntegerWidget(description="""Initial column number to sort by. Negative number gives descending sort."""),
              ),
    IntegerField('batchsize',
              required=1,
	      default=40,
              searchable=0,
              default_output_type='text/plain',
              allowable_content_types=('text/plain'),
              widget = IntegerWidget(description="""Number of lines to list per page."""),
              ),
    ), marshall=ScoutRFC822Marshaller())


def _colsort(x, y, reverse, sortformula):
#    print "compare "+str(x)+" and "+str(y)+" using "+sortformula
    o = x
    try:
        xval = eval(sortformula)
    except:
	xval = ''

    o = y
    try:
        yval = eval(sortformula)
    except:
	yval = ''

#    print "compare '"+str(xval)+"' and '"+str(yval)+"'"
    if isinstance(xval, int) and isinstance(yval, int):
        if reverse:
#	    print "reverse int comparison: "+str(xval)+" - "+str(yval)
	    return xval-yval
        else:
#	    print "int comparison: "+str(yval)+" - "+str(xval)
	    return yval-xval
    else:
	if reverse:
#	    print "reverse string comparison: "+str(yval).lower()+" ~ "+str(xval).lower()
	    return cmp(str(yval).lower(), str(xval).lower())
	else:
#	    print "string comparison: "+str(xval).lower()+" ~ "+str(yval).lower()
	    return cmp(str(xval).lower(), str(yval).lower())


def _reverse(x, y):
    return cmp(y, x)

class Listing(BaseContent, Translatable):
    """Listing configuration object"""
    schema = schema
    content_icon = 'images/list_icon.gif'
    _at_rename_after_creation = True

    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/listing_view',
        'permissions': (CMFCorePermissions.View,)
        },)

    def debug(self, msg):
        # self.plone_log(msg)
        pass

    def eyeshare_i18n(self, msgid, text=None):
        if not text:
            text = msgid
        return self.PTS_translate(msgid, text)
    

    def _makecolsort(self, reverse, sortformula):

        """Return a lambda function for use with sort implementing
        the specified sortformula, obeying the reverse flag for reverse
        sorting."""
        
#	print "makecolsort: sortformula="+sortformula
	return eval("lambda x,y: _colsort(x, y, "+str(reverse)+",'"+sortformula+"')")

    def column_fields(self, sortformulas=0):

        """Return the list of column formulas for the current listing."""

#	print "self.column_fields() called"
#	columnfield = self.getColumns().replace("\r", "")
        if hasattr(self, 'getRawColumns'):
            columnfield = self.getRawColumns().replace("\r", "")
            self.plone_log("using getRawColumns, columnfield="+str(columnfield))
        else:
            columnfield = self.getColumns().replace("\r", "")
            self.plone_log("using getColumns, columnfield="+str(columnfield))
	if columnfield.startswith("@"):
	    columnfield = defaults[columnfield[1:]]

	list = columnfield.split("\n")
#	print "column fields: "+str(list)
	columns = []
	maxnum = 0
	tmplist = []
	for line in list:
	    if len(line) > 0:
		tmplist = line.split("|")
		cnum = tmplist[0].lstrip()
		colnum = int(cnum)
		if colnum > maxnum:
			maxnum = colnum
	i = 0
	while i < maxnum:
		columns.append('')
		i = i + 1

	for line in list:
	    if len(line) > 0:
		tmplist = line.split("|")
#		print cnum
		if sortformulas and len(tmplist)>2:
		    columns[int(tmplist[0])-1] = tmplist[2]
		else:
		    columns[int(tmplist[0])-1] = tmplist[1]
#	print "column_fields: "+str(columns)
	return columns


    def listingdata(self, sindex=1, ssortcol='0', expand=None, scatcol=None, sbatchsize=None, nocat=0):
	
	"""Return data structure describing listing well enough that
        a page template can render it. Check out listing_view for a
        code-centric documentation of the returned structure. :-)"""

        
        
        self.debug("listingdata sindex="+str(sindex)+", ssortcol="+str(ssortcol)+", expand="+str(expand)+", scatcol="+str(scatcol)+", sbatchsize="+str(sbatchsize)+"\n")
	cindex = int(sindex)

        batchsize = None
        if sbatchsize:
            batchsize = int(sbatchsize)

        if not batchsize:
            batchsize = int(self.getBatchsize())
            
	generator = self.getGenerator()
	categorize = self.getCategories() # support only one level categorization, for now
	sortcol = int(ssortcol)
	context = aq_parent(self)

        catcol = 0
        if categorize.startswith("#"):
            catcol = int(categorize[1:])

        if scatcol:
            catcol = int(scatcol)

        # forced non-categorization? (this is getting downright ugly)
        if nocat:
            catcol = 0

#        self.debug("in listingdata, and catcol is "+str(catcol)+"\n")
        catdesc = 0
        ocatcol = catcol
        if catcol < 0:
            catcol = -catcol
            catdesc = 1

#        self.debug("after test in listingdata, and catcol is "+str(catcol)+"\n")
#        self.debug("after test in listingdata, and catdesc is "+str(catdesc)+"\n")

#	print "context: "+str(context)
	self.debug("Listing::listingdata generator: "+generator+"\n")
	self.debug("Listing::listingdata categorize: "+categorize+"\n")
	self.debug("Listing::listingdata catcol: "+str(catcol)+"\n")
#	self.debug("Listing::listingdata catcol: "+str(catcol)+"\n")

	# Get list of objects to list
	try:
	    objects = eval(generator)
	except Exception, info:
            self.plone_log("Listing.listingdata: exception raised when evaluating generator "+generator+": "+str(info))
            objects=str(info)
	    #objects = []
	if type(objects)==type("string"):
            errormessage = objects
            objects = []
        else:
            errormessage = None

#	self.debug("Listing::listingdata: objects are "+str(objects)+"\n")

	# If page template didn't specify a sort column, use the default value from
	# the listing object
	if sortcol == 0:
	    sortcol = int(self.getSortby())

	# set reverse flag if sort column is negative
	reverse = 0
	if sortcol < 0:
		sortcol = -sortcol
		reverse = 1

	columnformulas = self.column_fields(1)
        if sortcol > len(columnformulas)+1:
            raise "Invalid listing definition: sortcolumn greather than number of columns"
	sortformula = columnformulas[sortcol-1]

	if catcol:
	  self.debug("listingdata: catcol="+str(catcol)+", len(columnformulas)="+str(len(columnformulas))+"\n")
	  categorize = columnformulas[catcol-1]

	olist = []
	data = {}
        objcount = 0
	if catcol:
#            print "categorize: "+categorize
	    for o in objects:
		category = '(ingen)'
                categories = []
                categories = self.columnexpand(str(eval(categorize)))
                for category in categories:
                    if data.has_key(category):
                        data[category].append(o)
                    else:
                        data[category] = [o]
                
#	    print "categorized list: "+str(data)

	    datakeys = data.keys()
#            self.debug("before sort in listingdata, and catdesc is "+str(catdesc)+"\n")
            if catdesc:
#                self.debug("sorting descending\n")
                datakeys.sort(_reverse)
            else:
#                self.debug("sorting ascending\n")
                datakeys.sort()

	    for key in datakeys:
		if key == expand:
		    olist.append({'cat': 1, 'key': key, 'data': key+" ("+str(len(data[key]))+")", 'expanded': 1})
		    data[key].sort(self._makecolsort(reverse, sortformula))
		    for o in data[key]:
                        values = self.column_values(o)
                        for valueset in self.valuesets(values,expand,catcol):
                            olist.append({'data': valueset, 'cat': 0, 'o': o})
		else:
                    linkkey = key
                    # kludgy handling of &nbsp; in links
                    linkkey = linkkey.replace('&', '%26')
                    linkkey = linkkey.replace(';', '%3B')
                    olist.append({'cat': 1, 'key': key, 'linkkey': linkkey, 'data': key+" ("+str(len(data[key]))+")", 'expanded': 0})

            objcount = len(olist)
	else:
            objcount = len(objects)
	    objects.sort(self._makecolsort(reverse, sortformula))
            count = 0
	    for o in objects:
              if count < cindex-2:
                  olist.append({'dummy': 'yes'})
              else:
                  values = self.column_values(o)
                  for valueset in self.valuesets(values, expand, catcol):
                      olist.append({'data': valueset, 'cat': 0, 'o': o})

                  if len(olist) > cindex+batchsize-1:
                      break
              count = count + 1
#--- end test
		
	# If listing index is past end of object list, adjust index
	if int(cindex) > len(olist):
	    cindex = int(len(olist)/batchsize)*batchsize+1

	index = cindex-1
	rlist = []
	while (index < len(olist)) and (index < cindex+batchsize-1):
	    rlist.append(olist[index])
	    index = index + 1

	value = {}

        if reverse:
            value['sortcol'] = -sortcol
        else:
            value['sortcol'] = sortcol

	value['reverse'] = reverse
	value['objectdata'] = rlist
#	value['headers'] = self.header_fields(categorize=scatcol)
	value['headers'] = self.header_fields()
	value['columncount'] = len(value['headers'])
	if categorize:
	    if catcol:
	        value['groupname'] = (self.header_fields())[catcol-1]['text']
            value['group'] = ocatcol
	    value['categorized'] = 1
	else:
            value['group'] = ''
	    value['categorized'] = 0

        if errormessage:
            value['errormessage'] = errormessage

	# batching

	value['objcount'] = objcount

	if objcount > batchsize:
	    value['batched'] = 1
	    index = 1
	    batches = []
	    while index < objcount:
	        stop = index + batchsize - 1
	        if stop > objcount:
	            stop = objcount
	        current = 0
	        if index == cindex:
	    	    current = 1
	        batches.append({'index': index, 'name':str(index)+"-"+str(stop), 'current': current})
	        index = index + batchsize

            prev = cindex - batchsize
	    if prev < 1:
	        prev = None

	    next = cindex + batchsize
	    if next > objcount:
	        next = None

	    value['batchdata'] = {'batches': batches, 'prev': prev, 'curr': cindex, 'next': next}
	else:
	    value['batched'] = 0

        if hasattr(self, 'getButtons') and (self.getButtons().strip() != ''):
            value['buttons'] = self.getButtons()

        if hasattr(self, 'getType_name') and (self.getType_name().strip() != ''):
            value['type_name'] = self.getType_name()

        if hasattr(self, 'getObject_folder') and (self.getObject_folder().strip() != ''):
            value['post_url'] = self.portal_url()+"/"+self.getObject_folder()

	self.debug("Listing::listingdata returning: "+str(value)+"\n")
#	print "listingdata: returning "+str(value)
	return value

    def valuesets_wrong(self, values, expand, catcol):
        """Ugly awful hack to allow expansion of consultant list into separate cases.
        Only allows one 'expandable' column per listing."""
#        print "valuesets: values = "+str(values)
        result = []
        index = 0
        appended = 0
        for value in values:
            expanded = 0
            if value.startswith("EXPAND:"):
                items = value[value.index(":")+1:].split(",")
                for item in items:
                    if not catcol or (catcol == index+1 and item == expand):
                        valuescopy = values[:]
                        valuescopy[index] = item
                        result.append(valuescopy)
                        appended = 1
                        expanded = 1
                if not expanded:
                    valuescopy = values[:]
                    valuescopy[index] = value[value.index(":")+1:]
                    result.append(valuescopy)
                    appended = 1
            index = index + 1
        if not appended:
            result.append(values)
#        print "valuesets: result = "+str(result)
        return result

    def valuesets(self, values, expand, catcol):
        """Remove 'EXPANDABLE' prefix from listing display."""
#        print "valuesets: values = "+str(values)
        result = []
        index = 0
        appended = 0
        for value in values:
            expanded = 0
            if value.startswith("EXPAND:"):
                valuescopy = values[:]
                valuescopy[index] = value[value.index(":")+1:]
                result.append(valuescopy)
                appended = 1
            index = index + 1
        if not appended:
            result.append(values)
#        print "valuesets: result = "+str(result)
        return result

    def columnexpand(self, value):
        """Make expandable lists into several categories."""
        if value.startswith("EXPAND:"):
            return value[value.index(":")+1:].split(",")
        else:
            return [value]

    def column_values(self, o, categorize=None):

        """Return a list of the column values for the specified object."""

#	print "column_values called with "+str(o)
	context = aq_parent(self)
	values = []
	colnum = 1
	for column in self.column_fields():
	    if categorize and (int(categorize) == colnum):
		pass
	    else:
		if len(column) > 0:
		    try:
			values.append(str(eval(column)))
		    except Exception, args:
			values.append('')
#			values.append('[exception]')
			self.plone_log("Listing: exception caught when evaluating '"+str(column)+"'")
			self.plone_log("Listing: exception: "+str(args))
	    	else:
		    values.append("")
	    colnum = colnum + 1
	return values

    def header_fields(self, categorize=None):

        """Return the header fields for the current listing."""
        
	headerfield = self.getHeaders().replace("\r", "")

	if headerfield.startswith("@"):
	    headerfield = defaults[headerfield[1:]]

	list = headerfield.split("\n")
#	print "header fields: "+str(list)
	headers = []
        columnwidths = []
	maxnum = 0
	for line in list:
	    if len(line) > 0:
		tmplist = line.split("|")
		cnum = tmplist[0]
		colnum = int(cnum)
		if colnum > maxnum:
			maxnum = colnum
	i = 0
	while i < maxnum:
		headers.append('')
		columnwidths.append('')
		i = i + 1

	for line in list:
	    if len(line) > 0:
		tmplist = line.split("|")
#		print cnum
		headers[int(tmplist[0])-1] = tmplist[1]
                if len(tmplist) > 2:
                    columnwidths[int(tmplist[0])-1] = tmplist[2]

	headerinfo = []
	index = 1
	for header in headers:
	    if categorize and (int(categorize) == index):
		pass
	    else:
                headerinfo.append({'text':header, 'index':index, 'width': columnwidths[index-1]})
		index = index + 1

	return headerinfo

    def getObjects(self, type='*'):
        """Generic 'get these objects' utility function ment used
        as a generator function in a listing object."""
        
	portal_catalog = getToolByName(self, "portal_catalog")
#	print "whee! 2"
	if type=='*':
	  brains = portal_catalog.searchResults()
	else:
	  brains = portal_catalog.searchResults(portal_type=type)
	objects = []
	for brain in brains:
          try:
	     o = brain.getObject()
	     if o != None:
                 objects.append(o)
	  except:
	     pass
	return objects


registerType(Listing, PROJECTNAME)


