__author__  = 'Oberon IT <alle@oberon.no>'

"""
Based on Translateable.py from PloneCollectorNG
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName

from config import i18n_domain

class Translatable:
    """ Mix-in class to provide i18n support for FS-based code """

    security = ClassSecurityInfo()

    def _getPTS(self):
        """ return PTS instance """

        pts = getattr(self, '_v_have_pts', None)
        if pts is None:
            try:
                pts = self.Control_Panel.TranslationService
                self._v_have_pts = pts
            except:
                self._v_have_pts = None

        try:
            self._v_have_pts.translate
            return self._v_have_pts
        except:
            return None

    def _getPloneEncoding(self):
        """ return the site encoding of Plone """
        encoding = getattr(self, '_v_site_encoding', None)
        if encoding is None:
            self._v_site_encoding = self.portal_properties.site_properties.default_charset
        return self._v_site_encoding    


    security.declarePublic('PTS_translate')
    def PTS_translate(self, msgid, text, target_language=None, as_unicode=1,**kw):
        """ ATT: this code is subject to change """

        pts = self._getPTS()
        if pts is None:
            return self._interpolate(text, kw)

        # Using the as_unicode parameter requires the lastest PTS version
        # from the CVS. In generell we retrieve the translations from PTS
        # always as unicode and convert it back to the Plone site encoding
        # if necessary

        v = pts.translate(domain=i18n_domain, 
                            msgid=msgid, 
                            context=self,
                            mapping=kw,  
                            default=text,
                            target_language=target_language,
                            as_unicode=0)

        if as_unicode:
            return v
        else:
            return v.encode(self._getPloneEncoding())

    security.declarePublic('getLanguages')
    def getLanguages(self):
        """ return the languages """
        pts = self._getPTS()
        if pts:
            return pts.getLanguages(i18n_domain)
        else:
            return []

    def _interpolate(self, text, mapping):
        """ convert a string containing vars for interpolation ('$var')
            with the corresponding value from the mapping.
        """ 
        for k,v in mapping.items():
            text = text.replace('$%s' % k, str(v))
        return text
            
InitializeClass(Translatable)


